from django.test import TestCase

import code
import readline

from django_omise.models.schedule import Schedule, Occurrence
from django_omise.omise import omise
from django_omise.utils.core_utils import update_or_create_from_omise_object

from django_omise.tests.base import OmiseBaseTestCase

from unittest import mock

from .test_utils import mocked_base_schedule_request

# Create your tests here.
class CustomerTestCase(OmiseBaseTestCase):
    def setUp(self):
        self.customer = self.create_customer(id="test_customer_id")
        self.card = self.create_card(id="test_card_id")

    @mock.patch("requests.get", side_effect=mocked_base_schedule_request)
    def test_schedule_update_or_create(self, mock_get_schedule):
        omise_schedule = omise.Schedule.retrieve("test_schedule_id")
        schedule = update_or_create_from_omise_object(omise_object=omise_schedule)
        self.assertEqual(Schedule.objects.count(), 1)

    @mock.patch("requests.get", side_effect=mocked_base_schedule_request)
    def test_schedule_occurrence(self, mock_get_schedule):
        omise_schedule = omise.Schedule.retrieve("test_schedule_id")
        schedule = update_or_create_from_omise_object(omise_object=omise_schedule)
        self.assertEqual(
            schedule.occurrences.all().count(), len(omise_schedule.occurrences)
        )



# Generated by Django 3.2.11 on 2022-05-28 00:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_omise', '0004_auto_20220523_0847'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='charge',
            options={'permissions': [('issue_refund', 'Can issue refunds')]},
        ),
    ]



from django.test import TestCase
from django_omise.context_processors import omise_keys
from django.conf import settings

from django_omise.tests.mock_django import MockRequest

# Create your tests here.
class ContextProcessorTestCase(TestCase):
    def test_omise_public_key(self):
        self.assertEqual(
            omise_keys(request=MockRequest())["omise_public_key"],
            settings.OMISE_PUBLIC_KEY,
        )



base_schedule_response = """
{
  "object": "schedule",
  "id": "test_schedule_id",
  "deleted": false,
  "livemode": false,
  "location": "/schedules/test_schedule_id",
  "every": 1,
  "occurrences": {
    "object": "list",
    "data": [
      {
        "object": "occurrence",
        "livemode": false,
        "location": "/occurrences/test_occurrence_id",
        "id": "test_occurrence_id",
        "result": "chrg_test_5s63wg5cfhlc63ww0os",
        "schedule": "test_schedule_id",
        "message": null,
        "status": "successful",
        "processed_at": "2022-06-17T01:01:47Z",
        "created_at": "2022-06-17T01:01:47Z",
        "scheduled_on": "2022-06-17",
        "retry_on": null
      }
    ],
    "limit": 20,
    "offset": 0,
    "total": 1,
    "location": "/schedules/test_schedule_id/occurrences",
    "order": "chronological",
    "from": "1970-01-01T00:00:00Z",
    "to": "2022-06-17T01:02:04Z"
  },
  "on": {
    "days_of_month": [
      17
    ]
  },
  "in_words": "Every 1 month(s) on the 17th",
  "period": "month",
  "status": "running",
  "active": true,
  "charge": {
    "object": "scheduled_charge",
    "id": "test_schedule_charge_id",
    "livemode": false,
    "currency": "THB",
    "amount": 200000,
    "default_card": true,
    "card": "test_card_id",
    "customer": "test_customer_id",
    "description": "test charge",
    "metadata": {
    },
    "created_at": "2022-06-17T01:01:47Z"
  },
  "next_occurrences_on": [
    "2022-07-17",
    "2022-08-17",
    "2022-09-17",
    "2022-10-17",
    "2022-11-17",
    "2022-12-17",
    "2023-01-17",
    "2023-02-17",
    "2023-03-17",
    "2023-04-17",
    "2023-05-17",
    "2023-06-17"
  ],
  "ended_at": null,
  "start_on": "2022-06-17",
  "end_on": "2023-06-17",
  "created_at": "2022-06-17T01:01:47Z"
}"""

schedule_response_no_next_occurrences_on = """
{
  "object": "schedule",
  "id": "test_schedule_id",
  "deleted": false,
  "livemode": false,
  "location": "/schedules/test_schedule_id",
  "every": 1,
  "occurrences": {
    "object": "list",
    "data": [
      {
        "object": "occurrence",
        "livemode": false,
        "location": "/occurrences/test_occurrence_id",
        "id": "test_occurrence_id",
        "result": "chrg_test_5s63wg5cfhlc63ww0os",
        "schedule": "test_schedule_id",
        "message": null,
        "status": "successful",
        "processed_at": "2022-06-17T01:01:47Z",
        "created_at": "2022-06-17T01:01:47Z",
        "scheduled_on": "2022-06-17",
        "retry_on": null
      }
    ],
    "limit": 20,
    "offset": 0,
    "total": 1,
    "location": "/schedules/test_schedule_id/occurrences",
    "order": "chronological",
    "from": "1970-01-01T00:00:00Z",
    "to": "2022-06-17T01:02:04Z"
  },
  "on": {
    "days_of_month": [
      17
    ]
  },
  "in_words": "Every 1 month(s) on the 17th",
  "period": "month",
  "status": "running",
  "active": true,
  "charge": {
    "object": "scheduled_charge",
    "id": "test_schedule_charge_id",
    "livemode": false,
    "currency": "THB",
    "amount": 200000,
    "default_card": true,
    "card": "test_card_id",
    "customer": "test_customer_id",
    "description": "test charge",
    "metadata": {
    },
    "created_at": "2022-06-17T01:01:47Z"
  },
  "ended_at": null,
  "start_on": "2022-06-17",
  "end_on": "2023-06-17",
  "created_at": "2022-06-17T01:01:47Z"
}"""



from django.test import TestCase

from django.contrib.auth import get_user_model

from django_omise.models.core import Customer, Card, Charge
from django_omise.models.choices import Currency
from django_omise.omise import omise

from django_omise.utils.core_utils import update_or_create_from_omise_object

from unittest import mock

from .test_utils import (
    mocked_fully_refunded_charge_request,
    mocked_requests_post,
    mocked_requests_get,
    mocked_charge_with_card_request,
    mocked_partially_refunded_charge_request,
    mocked_fully_refunded_charge_request,
    mocked_base_charge_request,
    mocked_jpy_charge,
)

User = get_user_model()

# Create your tests here.
class ChargeTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            username="test_user1", email="test_user1@email.com"
        )

        self.post_patcher = mock.patch(
            "requests.post", side_effect=mocked_requests_post
        )
        self.get_patcher = mock.patch("requests.get", side_effect=mocked_requests_get)

        self.post_patcher.start()
        self.get_patcher.start()

        self.customer, created = Customer.get_or_create(user=self.user)
        self.customer.sync_cards()

    def tearDown(self):
        mock.patch.stopall()

    def test_no_charge_type_error(self):
        with self.assertRaises(
            ValueError, msg="At least a token, a card, or a source is required"
        ):
            Charge.charge(amount=100000, currency=Currency.THB)

    def test_more_than_one_charge_type_error(self):
        with self.assertRaises(
            ValueError, msg="Only one of token, card, or source must be specified"
        ):
            Charge.charge(amount=100000, currency=Currency.THB, token="123", card="123")

    @mock.patch("requests.post", side_effect=mocked_charge_with_card_request)
    @mock.patch("requests.get", side_effect=mocked_charge_with_card_request)
    def test_charge(self, mocked_post_request, mocked_get_request):
        charge = Charge.charge(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first(),
        )
        self.assertEqual(charge.amount, charge.get_omise_object().amount)
        self.assertEqual(charge.id, charge.get_omise_object().id)

    @mock.patch("requests.post", side_effect=mocked_charge_with_card_request)
    @mock.patch("requests.get", side_effect=mocked_charge_with_card_request)
    def test_charge_human_amount(self, mocked_post_request, mocked_get_request):
        unsaved_charge = Charge(amount=100000)
        self.assertEqual(unsaved_charge.human_amount, "1,000.00")

        charge = Charge.charge(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first(),
        )

        self.assertEqual(charge.human_amount, "1,000.00")

    @mock.patch("requests.post", side_effect=mocked_jpy_charge)
    @mock.patch("requests.get", side_effect=mocked_jpy_charge)
    def test_charge_human_amount_jpy(self, mocked_post_request, mocked_get_request):
        charge = Charge.charge(
            amount=100000,
            currency=Currency.JPY,
            card=self.customer.cards.live().first(),
        )

        self.assertEqual(charge.human_amount, "100,000.00")

    @mock.patch("requests.get", side_effect=mocked_partially_refunded_charge_request)
    def test_charge_extended_status_partially_refund(self, mocked_api):
        omise_charge = omise.Charge.retrieve("charge_id")
        charge = update_or_create_from_omise_object(omise_object=omise_charge)

        self.assertEqual(charge.extended_status, "partially refunded")

    @mock.patch("requests.get", side_effect=mocked_fully_refunded_charge_request)
    def test_charge_extended_status_fully_refund(self, mocked_api):
        omise_charge = omise.Charge.retrieve("charge_id")
        charge = update_or_create_from_omise_object(omise_object=omise_charge)

        self.assertEqual(charge.extended_status, "refunded")

    @mock.patch("requests.get", side_effect=mocked_base_charge_request)
    def test_charge_extended_status_normal(self, mocked_api):
        omise_charge = omise.Charge.retrieve("charge_id")
        charge = update_or_create_from_omise_object(omise_object=omise_charge)

        self.assertEqual(charge.extended_status, omise_charge.status)

    @mock.patch("django_omise.models.core.Charge.update_or_create_from_omise_object")
    @mock.patch("django_omise.models.core.omise.Charge.create")
    @mock.patch("requests.post", side_effect=mocked_base_charge_request)
    def test_create_charge_with_card_passed_arguments(
        self,
        mock_request,
        mock_omise,
        mock_update_or_create_method,
    ):

        charge = Charge.charge(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first(),
        )

        args, kwargs = mock_omise.call_args
        self.assertIn("customer", kwargs)
        self.assertIn("card", kwargs)

    @mock.patch("django_omise.models.core.Charge.update_or_create_from_omise_object")
    @mock.patch("django_omise.models.core.omise.Charge.create")
    @mock.patch("requests.post", side_effect=mocked_base_charge_request)
    def test_create_charge_with_token_as_string_passed_arguments(
        self,
        mock_request,
        mock_omise,
        mock_update_or_create_method,
    ):

        charge = Charge.charge(amount=100000, currency=Currency.THB, token="token_id")

        args, kwargs = mock_omise.call_args
        self.assertEqual(kwargs["card"], "token_id")

    @mock.patch("django_omise.models.core.Charge.update_or_create_from_omise_object")
    @mock.patch("django_omise.models.core.omise.Charge.create")
    @mock.patch("requests.post", side_effect=mocked_base_charge_request)
    @mock.patch("requests.get", side_effect=mocked_requests_get)
    def test_create_charge_with_token_as_token_instance_passed_arguments(
        self,
        mock_get_requests,
        mock_request,
        mock_omise,
        mock_update_or_create_method,
    ):

        token = omise.Token.retrieve("test_token_id")

        charge = Charge.charge(amount=100000, currency=Currency.THB, token=token)

        args, kwargs = mock_omise.call_args
        self.assertIn("card", kwargs)

    @mock.patch("django_omise.models.core.Charge.update_or_create_from_omise_object")
    @mock.patch("django_omise.models.core.omise.Charge.create")
    @mock.patch("requests.post", side_effect=mocked_base_charge_request)
    def test_create_charge_with_source_passed_arguments(
        self,
        mock_request,
        mock_omise,
        mock_update_or_create_method,
    ):

        charge = Charge.charge(
            amount=100000, currency=Currency.THB, source={"type": "promptpay"}
        )

        args, kwargs = mock_omise.call_args
        self.assertIn("source", kwargs)



base_charge_response = """{"object":"charge","id":"chrg_test_5s1kvbjga85m8a8rwu2","location":"/charges/chrg_test_5s1kvbjga85m8a8rwu2","amount":100000,"net":96094,"fee":3650,"fee_vat":256,"interest":0,"interest_vat":0,"funding_amount":100000,"refunded_amount":0,"transaction_fees":{"fee_flat":"0.0","fee_rate":"3.65","vat_rate":"7.0"},"platform_fee":{"fixed":null,"amount":null,"percentage":null},"currency":"THB","funding_currency":"THB","ip":null,"refunds":{"object":"list","data":[],"limit":20,"offset":0,"total":0,"location":"/charges/chrg_test_5s1kvbjga85m8a8rwu2/refunds","order":"chronological","from":"1970-01-01T00:00:00Z","to":"2022-06-05T11:09:02Z"},"link":null,"description":"Charge for order 3947","metadata":{},"card":{"object":"card","id":"card_test_5s1jzgw7oda499o8k0y","livemode":false,"location":"/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzgw7oda499o8k0y","deleted":false,"street1":null,"street2":null,"city":"Bangkok","state":null,"phone_number":null,"postal_code":"10320","country":"us","financing":"credit","bank":"JPMORGAN CHASE BANK N.A.","brand":"Visa","fingerprint":"7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=","first_digits":null,"last_digits":"4242","name":"JOHN DOE","expiration_month":6,"expiration_year":2024,"security_code_check":true,"tokenization_method":null,"created_at":"2022-06-05T09:38:32Z"},"source":null,"schedule":null,"customer":"cust_test_5s1jz157366mu6wr0ng","dispute":null,"transaction":"trxn_test_5s1kvbn1zds49714yo4","failure_code":null,"failure_message":null,"status":"successful","authorize_uri":"https://api.omise.co/payments/paym_test_5s1kvbjhn176xynlizh/authorize","return_uri":"http://www.example.com/orders/3947/complete","created_at":"2022-06-05T11:09:01Z","paid_at":"2022-06-05T11:09:01Z","expires_at":"2022-06-12T11:09:01Z","expired_at":null,"reversed_at":null,"zero_interest_installments":false,"branch":null,"terminal":null,"device":null,"authorized":true,"capturable":false,"capture":true,"disputable":true,"livemode":false,"refundable":true,"reversed":false,"reversible":false,"voided":false,"paid":true,"expired":false}"""

base_charge_with_metadata_response = """{"object":"charge","id":"chrg_test_5s1kvbjga85m8a8rwu2","location":"/charges/chrg_test_5s1kvbjga85m8a8rwu2","amount":100000,"net":96094,"fee":3650,"fee_vat":256,"interest":0,"interest_vat":0,"funding_amount":100000,"refunded_amount":0,"transaction_fees":{"fee_flat":"0.0","fee_rate":"3.65","vat_rate":"7.0"},"platform_fee":{"fixed":null,"amount":null,"percentage":null},"currency":"THB","funding_currency":"THB","ip":null,"refunds":{"object":"list","data":[],"limit":20,"offset":0,"total":0,"location":"/charges/chrg_test_5s1kvbjga85m8a8rwu2/refunds","order":"chronological","from":"1970-01-01T00:00:00Z","to":"2022-06-05T11:09:02Z"},"link":null,"description":"Charge for order 3947","metadata":{"data": "data"},"card":{"object":"card","id":"card_test_5s1jzgw7oda499o8k0y","livemode":false,"location":"/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzgw7oda499o8k0y","deleted":false,"street1":null,"street2":null,"city":"Bangkok","state":null,"phone_number":null,"postal_code":"10320","country":"us","financing":"credit","bank":"JPMORGAN CHASE BANK N.A.","brand":"Visa","fingerprint":"7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=","first_digits":null,"last_digits":"4242","name":"JOHN DOE","expiration_month":6,"expiration_year":2024,"security_code_check":true,"tokenization_method":null,"created_at":"2022-06-05T09:38:32Z"},"source":null,"schedule":null,"customer":"cust_test_5s1jz157366mu6wr0ng","dispute":null,"transaction":"trxn_test_5s1kvbn1zds49714yo4","failure_code":null,"failure_message":null,"status":"successful","authorize_uri":"https://api.omise.co/payments/paym_test_5s1kvbjhn176xynlizh/authorize","return_uri":"http://www.example.com/orders/3947/complete","created_at":"2022-06-05T11:09:01Z","paid_at":"2022-06-05T11:09:01Z","expires_at":"2022-06-12T11:09:01Z","expired_at":null,"reversed_at":null,"zero_interest_installments":false,"branch":null,"terminal":null,"device":null,"authorized":true,"capturable":false,"capture":true,"disputable":true,"livemode":false,"refundable":true,"reversed":false,"reversible":false,"voided":false,"paid":true,"expired":false}"""

base_charge_jpy = """{"object":"charge","id":"chrg_test_5s1l8lhaafl9d4aaa7o","location":"/charges/chrg_test_5s1l8lhaafl9d4aaa7o","amount":100000,"net":2427668,"fee":92211,"fee_vat":6455,"interest":0,"interest_vat":0,"funding_amount":2526334,"refunded_amount":0,"transaction_fees":{"fee_flat":"0.0","fee_rate":"3.65","vat_rate":"7.0"},"platform_fee":{"fixed":null,"amount":null,"percentage":null},"currency":"JPY","funding_currency":"THB","ip":null,"refunds":{"object":"list","data":[],"limit":20,"offset":0,"total":0,"location":"/charges/chrg_test_5s1l8lhaafl9d4aaa7o/refunds","order":"chronological","from":"1970-01-01T00:00:00Z","to":"2022-06-05T11:46:45Z"},"link":null,"description":"Charge for order 3947","metadata":{},"card":{"object":"card","id":"card_test_5s1jzgw7oda499o8k0y","livemode":false,"location":"/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzgw7oda499o8k0y","deleted":false,"street1":null,"street2":null,"city":"Bangkok","state":null,"phone_number":null,"postal_code":"10320","country":"us","financing":"credit","bank":"JPMORGAN CHASE BANK N.A.","brand":"Visa","fingerprint":"7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=","first_digits":null,"last_digits":"4242","name":"JOHN DOE","expiration_month":6,"expiration_year":2024,"security_code_check":true,"tokenization_method":null,"created_at":"2022-06-05T09:38:32Z"},"source":null,"schedule":null,"customer":"cust_test_5s1jz157366mu6wr0ng","dispute":null,"transaction":"trxn_test_5s1l8lly1hoehgljhva","failure_code":null,"failure_message":null,"status":"successful","authorize_uri":"https://api.omise.co/payments/paym_test_5s1l8lhds9x6b5dw6k0/authorize","return_uri":"http://www.example.com/orders/3947/complete","created_at":"2022-06-05T11:46:44Z","paid_at":"2022-06-05T11:46:44Z","expires_at":"2022-06-12T11:46:43Z","expired_at":null,"reversed_at":null,"zero_interest_installments":false,"branch":null,"terminal":null,"device":null,"authorized":true,"capturable":false,"capture":true,"disputable":true,"livemode":false,"refundable":true,"reversed":false,"reversible":false,"voided":false,"paid":true,"expired":false}"""

partially_refunded_response = """{"object":"charge","id":"test_charge_id","location":"/charges/test_charge_id","amount":100000,"net":96094,"fee":3650,"fee_vat":256,"interest":0,"interest_vat":0,"funding_amount":100000,"refunded_amount":50000,"transaction_fees":{"fee_flat":"0.0","fee_rate":"3.65","vat_rate":"7.0"},"platform_fee":{"fixed":null,"amount":null,"percentage":null},"currency":"THB","funding_currency":"THB","ip":null,"refunds":{"object":"list","data":[{"object":"refund","id":"rfnd_test_5s1kvj7i16l6lrqco9c","location":"/charges/test_charge_id/refunds/rfnd_test_5s1kvj7i16l6lrqco9c","livemode":false,"voided":false,"currency":"THB","amount":50000,"metadata":{},"charge":"test_charge_id","terminal":null,"transaction":"trxn_test_5s1kvj7lln9kh4v8k3d","status":"closed","funding_amount":50000,"funding_currency":"THB","created_at":"2022-06-05T11:09:37Z"}],"limit":20,"offset":0,"total":1,"location":"/charges/test_charge_id/refunds","order":"chronological","from":"1970-01-01T00:00:00Z","to":"2022-06-05T11:09:45Z"},"link":null,"description":"Charge for order 3947","metadata":{},"card":{"object":"card","id":"card_test_5s1jzgw7oda499o8k0y","livemode":false,"location":"/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzgw7oda499o8k0y","deleted":false,"street1":null,"street2":null,"city":"Bangkok","state":null,"phone_number":null,"postal_code":"10320","country":"us","financing":"credit","bank":"JPMORGAN CHASE BANK N.A.","brand":"Visa","fingerprint":"7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=","first_digits":null,"last_digits":"4242","name":"JOHN DOE","expiration_month":6,"expiration_year":2024,"security_code_check":true,"tokenization_method":null,"created_at":"2022-06-05T09:38:32Z"},"source":null,"schedule":null,"customer":"cust_test_5s1jz157366mu6wr0ng","dispute":null,"transaction":"trxn_test_5s1kvbn1zds49714yo4","failure_code":null,"failure_message":null,"status":"successful","authorize_uri":"https://api.omise.co/payments/paym_test_5s1kvbjhn176xynlizh/authorize","return_uri":"http://www.example.com/orders/3947/complete","created_at":"2022-06-05T11:09:01Z","paid_at":"2022-06-05T11:09:01Z","expires_at":"2022-06-12T11:09:01Z","expired_at":null,"reversed_at":null,"zero_interest_installments":false,"branch":null,"terminal":null,"device":null,"authorized":true,"capturable":false,"capture":true,"disputable":true,"livemode":false,"refundable":true,"reversed":false,"reversible":false,"voided":false,"paid":true,"expired":false}"""

fully_refunded_response = """{"object":"charge","id":"test_charge_id","location":"/charges/test_charge_id","amount":100000,"net":96094,"fee":3650,"fee_vat":256,"interest":0,"interest_vat":0,"funding_amount":100000,"refunded_amount":100000,"transaction_fees":{"fee_flat":"0.0","fee_rate":"3.65","vat_rate":"7.0"},"platform_fee":{"fixed":null,"amount":null,"percentage":null},"currency":"THB","funding_currency":"THB","ip":null,"refunds":{"object":"list","data":[{"object":"refund","id":"rfnd_test_5s1kvj7i16l6lrqco9c","location":"/charges/test_charge_id/refunds/rfnd_test_5s1kvj7i16l6lrqco9c","livemode":false,"voided":false,"currency":"THB","amount":50000,"metadata":{},"charge":"test_charge_id","terminal":null,"transaction":"trxn_test_5s1kvj7lln9kh4v8k3d","status":"closed","funding_amount":50000,"funding_currency":"THB","created_at":"2022-06-05T11:09:37Z"},{"object":"refund","id":"rfnd_test_5s1kvp13mtlmgaftih6","location":"/charges/test_charge_id/refunds/rfnd_test_5s1kvp13mtlmgaftih6","livemode":false,"voided":false,"currency":"THB","amount":50000,"metadata":{},"charge":"test_charge_id","terminal":null,"transaction":"trxn_test_5s1kvp16x977nh39ivs","status":"closed","funding_amount":50000,"funding_currency":"THB","created_at":"2022-06-05T11:10:05Z"}],"limit":20,"offset":0,"total":2,"location":"/charges/test_charge_id/refunds","order":"chronological","from":"1970-01-01T00:00:00Z","to":"2022-06-05T11:10:14Z"},"link":null,"description":"Charge for order 3947","metadata":{},"card":{"object":"card","id":"card_test_5s1jzgw7oda499o8k0y","livemode":false,"location":"/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzgw7oda499o8k0y","deleted":false,"street1":null,"street2":null,"city":"Bangkok","state":null,"phone_number":null,"postal_code":"10320","country":"us","financing":"credit","bank":"JPMORGAN CHASE BANK N.A.","brand":"Visa","fingerprint":"7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=","first_digits":null,"last_digits":"4242","name":"JOHN DOE","expiration_month":6,"expiration_year":2024,"security_code_check":true,"tokenization_method":null,"created_at":"2022-06-05T09:38:32Z"},"source":null,"schedule":null,"customer":"cust_test_5s1jz157366mu6wr0ng","dispute":null,"transaction":"trxn_test_5s1kvbn1zds49714yo4","failure_code":null,"failure_message":null,"status":"successful","authorize_uri":"https://api.omise.co/payments/paym_test_5s1kvbjhn176xynlizh/authorize","return_uri":"http://www.example.com/orders/3947/complete","created_at":"2022-06-05T11:09:01Z","paid_at":"2022-06-05T11:09:01Z","expires_at":"2022-06-12T11:09:01Z","expired_at":null,"reversed_at":null,"zero_interest_installments":false,"branch":null,"terminal":null,"device":null,"authorized":true,"capturable":false,"capture":true,"disputable":false,"livemode":false,"refundable":false,"reversed":false,"reversible":false,"voided":false,"paid":true,"expired":false}"""

charge_with_schedule_response = """{"object":"charge","id":"test_charge_id","location":"/charges/test_charge_id","amount":200000,"net":192189,"fee":7300,"fee_vat":511,"interest":0,"interest_vat":0,"funding_amount":200000,"refunded_amount":0,"transaction_fees":{"fee_flat":"0.0","fee_rate":"3.65","vat_rate":"7.0"},"platform_fee":{"fixed":null,"amount":null,"percentage":null},"currency":"THB","funding_currency":"THB","ip":null,"refunds":{"object":"list","data":[],"limit":20,"offset":0,"total":0,"location":"/charges/test_charge_id/refunds","order":"chronological","from":"1970-01-01T00:00:00Z","to":"2022-06-17T11:24:08Z"},"link":null,"description":null,"metadata":{},"card":{"object":"card","id":"card_test_5rwur2pj0oa7blxex9x","livemode":false,"location":"/customers/test_customer_id/cards/card_test_5rwur2pj0oa7blxex9x","deleted":false,"street1":null,"street2":null,"city":null,"state":null,"phone_number":null,"postal_code":null,"country":"us","financing":"credit","bank":"JPMORGAN CHASE BANK N.A.","brand":"Visa","fingerprint":"oOHg2QCA2wKkUXIDCVLVspG+u7n/xv9ogrvZ9xtZhcQ=","first_digits":null,"last_digits":"1111","name":"James","expiration_month":10,"expiration_year":2030,"security_code_check":true,"tokenization_method":null,"created_at":"2022-05-24T09:11:32Z"},"source":null,"schedule":"schd_test_5s67suxlifb0r6vzqar","customer":"test_customer_id","dispute":null,"transaction":"trxn_test_5s67sv1zzy51fun0vn9","failure_code":null,"failure_message":null,"status":"successful","authorize_uri":null,"return_uri":null,"created_at":"2022-06-17T07:40:39Z","paid_at":"2022-06-17T07:40:39Z","expires_at":"2022-06-24T07:40:39Z","expired_at":null,"reversed_at":null,"zero_interest_installments":false,"branch":null,"terminal":null,"device":null,"authorized":true,"capturable":false,"capture":true,"disputable":true,"livemode":false,"refundable":true,"reversed":false,"reversible":false,"voided":false,"paid":true,"expired":false}"""



from django.test import TestCase
from django.utils import timezone


from django_omise.models.core import Customer
from django_omise.models.schedule import Schedule

# Create your tests here.
class ManagerTestCase(TestCase):
    def setUp(self):
        self.create_customer()
        self.create_schedule()

    def create_customer(self):
        self.customer = Customer.objects.create(id="test_customer_id", livemode=False)

    def create_schedule(self):
        self.schedule = Schedule.objects.create(
            id="test_schedule_id",
            livemode=False,
            active=True,
            start_on=timezone.now(),
            end_on=timezone.now(),
        )

    def test_live_queryset(self):
        self.assertTrue(Customer.objects.live().filter(id=self.customer.id).exists())
        self.assertFalse(
            Customer.objects.deleted().filter(id=self.customer.id).exists()
        )

    def test_deleted_queryset(self):
        self.customer.deleted = True
        self.customer.save()
        self.assertTrue(Customer.objects.deleted().filter(id=self.customer.id).exists())
        self.assertFalse(Customer.objects.live().filter(id=self.customer.id).exists())

    def test_not_deleted_queryset(self):
        self.customer.deleted = True
        self.customer.save()

        self.assertTrue(Customer.objects.deleted().filter(id=self.customer.id).exists())
        self.assertFalse(
            Customer.objects.not_deleted().filter(id=self.customer.id).exists()
        )

    def test_deletable_manager_not_deleted_queryset(self):
        self.assertTrue(
            Schedule.objects.not_deleted().filter(id=self.schedule.id).exists()
        )

    def test_deletable_manager_live_queryset(self):
        self.assertTrue(Schedule.objects.live().filter(id=self.schedule.id).exists())

    def test_deletable_manager_deleted_queryset(self):
        self.schedule.deleted = True
        self.schedule.save()
        self.assertFalse(Schedule.objects.live().filter(id=self.schedule.id).exists())
        self.assertTrue(Schedule.objects.deleted().filter(id=self.schedule.id).exists())



schedule_with_one_charge_event_response = """
{
  "object": "event",
  "id": "test_event_id",
  "livemode": false,
  "location": "/events/test_event_id",
  "webhook_deliveries": [],
  "data": {
    "object": "schedule",
    "id": "test_schedule_id",
    "deleted": false,
    "livemode": false,
    "location": "/schedules/test_schedule_id",
    "every": 1,
    "occurrences": {
      "object": "list",
      "data": [],
      "limit": 20,
      "offset": 0,
      "total": 0,
      "location": "/schedules/test_schedule_id/occurrences",
      "order": "chronological",
      "from": "1970-01-01T07:00:00+07:00",
      "to": "2022-06-17T08:40:54+07:00"
    },
    "on": {
      "days_of_month": [
        17
      ]
    },
    "in_words": "Every 1 month(s) on the 17th",
    "period": "month",
    "status": "running",
    "active": true,
    "charge": {
      "object": "scheduled_charge",
      "id": "test_schedule_charge_id",
      "livemode": false,
      "currency": "THB",
      "amount": 100000,
      "default_card": false,
      "card": "test_card_id",
      "customer": "test_customer_id",
      "description": "",
      "metadata": {},
      "created_at": "2022-06-17T08:40:54+07:00"
    },
    "next_occurrences_on": [
      "2022-06-17",
      "2022-07-17",
      "2022-08-17",
      "2022-09-17",
      "2022-10-17",
      "2022-11-17",
      "2022-12-17",
      "2023-01-17",
      "2023-02-17",
      "2023-03-17",
      "2023-04-17",
      "2023-05-17",
      "2023-06-17"
    ],
    "ended_at": null,
    "start_on": "2022-06-17",
    "end_on": "2023-06-17",
    "created_at": "2022-06-17T08:40:54+07:00"
  },
  "key": "schedule.create",
  "created_at": "2022-06-17T01:40:54Z"
}"""



from django_omise.models.schedule import Schedule
from django_omise.omise import omise
from django_omise.utils.core_utils import update_or_create_from_omise_object

from django_omise.tests.base import OmiseBaseTestCase

from unittest import mock

from .test_utils import mocked_schedule_event_request

# Create your tests here.
class EventTestCase(OmiseBaseTestCase):
    def setUp(self):
        self.customer = self.create_customer(id="test_customer_id")
        self.card = self.create_card(id="test_card_id")

    @mock.patch("requests.get", side_effect=mocked_schedule_event_request)
    def test_schedule_event(self, mock_schedule_event):
        event = omise.Event.retrieve("test_event_id")
        event_data = event.data
        schedule = update_or_create_from_omise_object(omise_object=event_data)
        self.assertTrue(Schedule.objects.filter(id=schedule.id).exists())



from django.test import TestCase

from django.contrib.auth import get_user_model

from django_omise.models.core import Customer, Card

from unittest import mock

from .test_utils import (
    mocked_requests_post,
    mocked_requests_get,
    mocked_add_card_request,
    mocked_delete_card_request,
)

User = get_user_model()

# Create your tests here.
class CardTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            username="test_user1", email="test_user1@email.com"
        )

        self.post_patcher = mock.patch(
            "requests.post", side_effect=mocked_requests_post
        )
        self.get_patcher = mock.patch("requests.get", side_effect=mocked_requests_get)

        self.post_patcher.start()
        self.get_patcher.start()

    def tearDown(self):
        mock.patch.stopall()

    def test_card_str(self):
        customer, created = Customer.get_or_create(user=self.user)
        customer.sync_cards()

        self.assertIn(
            customer.cards.live().first().last_digits,
            str(customer.cards.live().first()),
        )

    @mock.patch("requests.delete", side_effect=mocked_delete_card_request)
    def test_delete_card(self, mocked_request):
        customer, created = Customer.get_or_create(user=self.user)
        customer.sync_cards()

        all_cards_count = Card.objects.count()
        Card.objects.first().delete()

        self.assertEqual(Card.objects.count() - 1, Card.objects.live().count())

        self.assertEqual(all_cards_count, Card.objects.count())



from django.contrib.admin.sites import AdminSite
from django.contrib.auth import get_user_model

from django_omise.admin import CardInline
from django_omise.models.core import Customer

from django_omise.tests.base import ClientAndUserBaseTestCase, OmiseBaseTestCase
from django_omise.tests.mock_django import MockRequest

User = get_user_model()


class AdminCardTestCase(ClientAndUserBaseTestCase, OmiseBaseTestCase):
    def test_card_inline_has_change_permission(self):
        card_inline = CardInline(parent_model=Customer, admin_site=AdminSite())
        request = MockRequest(user=self.user)
        self.assertFalse(card_inline.has_change_permission(request=request))

    def test_card_inline_is_not_default_card(self):

        card_inline = CardInline(parent_model=Customer, admin_site=AdminSite())
        card = self.create_card()
        self.assertFalse(card_inline.default_card(obj=card))

    def test_card_inline_is_default_card(self):

        card_inline = CardInline(parent_model=Customer, admin_site=AdminSite())
        card = self.create_card()
        customer = self.create_customer()

        card.customer = customer
        card.save()

        customer.default_card = card
        customer.save()

        self.assertTrue(card_inline.default_card(obj=card))
